import json

from django.utils.module_loading import import_string

from .app_settings import app_settings

class Backend(object):
    def send(self, url, data):
        raise NotImplementedError()

    def validate(self, content_type, content):
        if content_type.startswith('application/json'):
            result = json.loads(content)

            if not result['ok']:
                raise ValueError(result['error'])

        elif content != 'ok':
            raise ValueError(content)

def get_backend():
    """
    Wrap the backend in a function to not load it at import time.
    get_backend() caches the backend on first call.
    """
    if get_backend.backend is None:
        get_backend.backend = import_string(app_settings.BACKEND)()
    return get_backend.backend
get_backend.backend = None



import json

from django.conf import settings
from django.template.loader import render_to_string

from .utils import get_backend
from .app_settings import app_settings

def slack_message(template, context=None, attachments=None, fail_silently=None):
    backend = get_backend()
    data = {}
    context = dict(context or {}, settings=settings)
    if fail_silently is None:
        fail_silently = app_settings.FAIL_SILENTLY

    for k, v in {
        'text': {
            'default': '',
            'required': False, # Checked later
        },
        'token': {
            'default': app_settings.TOKEN,
            'required': True,
        },
        'channel': {
            'default': app_settings.CHANNEL,
            'required': False, # Checked later
        },
        'icon_url': {
            'default': app_settings.ICON_URL,
            'required': False,
        },
        'icon_emoji': {
            'default': app_settings.ICON_EMOJI,
            'required': False,
        },
        'username': {
            'default': app_settings.USERNAME,
            'required': False,
        },
        'attachments': {
            'default': attachments,
            'render': False,
            'required': False,
        },
        'endpoint_url': {
            'default': app_settings.ENDPOINT_URL,
            'render': True,
            'required': False,
        },
    }.items():
        # First, set from default if we have one
        if v['default']:
            data[k] = v['default']

        # Render template if necessary
        if v.get('render', True):
            try:
                val = render_to_string(template, dict(
                    context,
                    django_slack='django_slack/%s' % k,
                )).strip().encode('utf8', 'ignore')
            except Exception:
                if fail_silently:
                    return
                raise

            if val:
                data[k] = val

        # Check if paramater is required
        if v['required']:
            if data.get(k, None):
                continue

            if fail_silently:
                return

            raise ValueError("Missing or empty required parameter: %s" % k)

    if 'text' not in data and 'attachments' not in data:
        raise ValueError("text parameter is required if attachments is not set")

    # The endpoint URL is not part of the data payload but as we construct it
    # within `data` we must remove it.
    endpoint_url = data.pop('endpoint_url')

    # If a custom endpoint URL was specified then we need to wrap it, otherwise
    # we need to ensure attachments are encoded.
    if endpoint_url == app_settings.DEFAULT_ENDPOINT_URL:
        # As a special case, if a custom endpoint is not set (eg. for a
        # private channel), then the channel parameter is not required.
        if not data.get('channel'):
            if fail_silently:
                return

            raise ValueError("channel parameter is required if custom " \
                "endpoint URL is not specified")

        if 'attachments' in data:
            data['attachments'] = json.dumps(data['attachments'])
    else:
        data = {'payload': json.dumps(data)}

    try:
        backend.send(endpoint_url, data)
    except Exception:
        if not fail_silently:
            raise



from celery import task

from django.utils.module_loading import import_string

from .app_settings import app_settings

@task
def send(*args, **kwargs):
    import_string(app_settings.BACKEND_FOR_QUEUE)().send(*args, **kwargs)



from django.conf import settings
from django.test import TestCase, override_settings

from django_slack import slack_message
from django_slack.utils import get_backend
from django_slack.app_settings import app_settings

from tests.backends import RaisingBackend

class TestOverride(TestCase):
    def setUp(self):
        # Fake the backend not having been initialized yet.
        get_backend.backend = None

    def test_override(self):
        """
        Ensure that accessing a setting in an override works.
        """
        DISABLED_BACKEND = 'django_slack.backends.DisabledBackend'
        with override_settings(SLACK_BACKEND=DISABLED_BACKEND):
            self.assertEqual(settings.SLACK_BACKEND, DISABLED_BACKEND)
            self.assertEqual(app_settings.BACKEND, DISABLED_BACKEND)

    def test_backend_cache(self):
        """
        Ensure that the backend is cached once it is called once.
        """
        self.assertEqual(id(get_backend()), id(get_backend()))

    def test_backend_override(self):
        """
        Ensure the backend can be overridden.
        """
        with override_settings(SLACK_BACKEND='tests.backends.RaisingBackend'):
            with self.assertRaises(RaisingBackend.RaisedException):
                slack_message('test.slack', {'text': 'test'})

    def test_fail_silently(self):
        """
        Ensure fail silently can be overridden.
        """
        # Note that this will fail if test_backend_override ever fails.
        with override_settings(
            SLACK_FAIL_SILENTLY=True,
            SLACK_BACKEND='tests.backends.RaisingBackend',
        ):
            slack_message('test.slack', {'text': 'test'})



import pprint

from six.moves import urllib

from django.utils.module_loading import import_string

from .utils import Backend
from .app_settings import app_settings

class UrllibBackend(Backend):
    def send(self, url, data):
        r = urllib.request.urlopen(urllib.request.Request(
            url,
            urllib.parse.urlencode(data).encode('utf-8'),
        ))

        result = r.read().decode('utf-8')

        self.validate(r.headers['content-type'], result)

class RequestsBackend(Backend):
    def __init__(self):
        # Lazily import to avoid dependency
        import requests

        self.session = requests.Session()

    def send(self, url, data):
        r = self.session.post(url, data=data, verify=False)

        self.validate(r.headers['Content-Type'], r.text)

class ConsoleBackend(Backend):
    def send(self, url, data):
        print("I: Slack message:")
        pprint.pprint(data, indent=4)
        print("-" * 79)

class DisabledBackend(Backend):
    def send(self, url, data):
        pass

class CeleryBackend(Backend):
    def __init__(self):
        # Lazily import to avoid dependency
        from .tasks import send
        self._send = send

        # Check we can import our specified backend up-front
        import_string(app_settings.BACKEND_FOR_QUEUE)()

    def send(self, *args, **kwargs):
        # Send asynchronously via Celery
        self._send.delay(*args, **kwargs)

Urllib2Backend = UrllibBackend # For backwards-compatibility



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django-slack',

    url="https://chris-lamb.co.uk/projects/django-slack",
    version='5.2.3',
    description="Provides easy-to-use integration between Django projects and "
        "the Slack group chat and IM tool.",

    author="Chris Lamb",
    author_email="chris@chris-lamb.co.uk",
    license="BSD",

    packages=find_packages(),
    include_package_data=True,
    install_requires=(
        'Django>=1.8.0',
        'requests',
        'six',
    ),
)
